@@ -1,15 +1,23 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
-#include "config.h"
-#include <dune/common/parallel/indexset.hh>
-#include <dune/common/parallel/communicator.hh>
-#include <dune/common/parallel/remoteindices.hh>
-#include <dune/common/enumset.hh>
+
+#include <config.h>
+
 #include <algorithm>
 #include <iostream>
+#include <ostream>
+#include <string>
+#include <vector>
 
-#if HAVE_MPI
-#include "mpi.h"
+#include <mpi.h>
+
+#include <dune/common/enumset.hh>
+#include <dune/common/parallel/communicator.hh>
+#include <dune/common/parallel/indexset.hh>
+#include <dune/common/parallel/interface.hh>
+#include <dune/common/parallel/plocalindex.hh>
+#include <dune/common/parallel/remoteindices.hh>
+#include <dune/common/unused.hh>
 
 enum GridFlags {
   owner, overlap, border
@@ -668,11 +676,9 @@ void MPI_err_handler(MPI_Comm *comm, int *err_code, ...){
   delete[] err_string;
   throw MPIError(s, *err_code);
 }
-#endif // HAVE_MPI
 
 int main(int argc, char **argv)
 {
-#if HAVE_MPI
   MPI_Init(&argc, &argv);
   MPI_Errhandler handler;
   MPI_Comm_create_errhandler(MPI_err_handler, &handler);
@@ -714,7 +720,4 @@ int main(int argc, char **argv)
   MPI_Finalize();
 
   return 0;
-#else
-  return 77;
-#endif
 }
