@@ -14,6 +14,26 @@
 
 #include <dune/common/std/optional.hh>
 #include <dune/common/test/testsuite.hh>
+#include <dune/common/unused.hh>
+
+class T {
+public:
+
+  T() : valid_(true) {}
+
+  T(const T& other) : valid_(true) {}
+
+  T(T&& other) : valid_(true) { other.valid_ = false; }
+
+  T& operator=(const T& other) { valid_ = true; return *this; }
+
+  T& operator=(T&& other) { valid_ = true; other.valid_ = false; return *this; }
+
+  bool valid() const { return valid_; }
+
+protected:
+  bool valid_;
+};
 
 Dune::Std::optional< std::string > create ( bool b )
 {
@@ -53,8 +73,27 @@ int main()
   test.check( !optFalse );
   test.check( *optTrue == "foo" );
 
+  // Check construction of optional for const types
+  Dune::Std::optional<const int> o1(42);
+  Dune::Std::optional<const int> DUNE_UNUSED o2(o1);
+  Dune::Std::optional<const int> DUNE_UNUSED o3(std::move(o1));
+
   optFalse = create( true );
   test.check( *optFalse == "void" );
 
+  {
+    T t1;
+    Dune::Std::optional<T> o(t1);
+    test.check(t1.valid());
+
+    T t2;
+    o = t2;
+    test.check(t2.valid());
+
+    T t3;
+    o = std::move(t3);
+    test.check(not(t3.valid()));
+  }
+
   return test.exit();
 }
